PROGRAM_NAME='PANASONIC, AG6040, RS-232, BASIC, 4-95, BA'
(*   DATE:05/20/99    TIME:16:17:14    *)
(***********************************************************)
(* The following code block is provided as a guide to      *)
(* programming the device(s) listed above. This is a       *)
(* sample only, and will most likely require modification  *)
(* to be integrated into a master program.                 *)
(*                                                         *)
(* Device-specific protocols should be obtained directly   *)
(* from the equipment manufacturer to ensure compliance    *)
(* with the most current versions. Within the limits       *)
(* imposed by any non-disclosure agreements, AMX will      *)
(* provide protocols upon request.                         *)
(*                                                         *)
(* If further programming assistance is required, please   *)
(* contact your AMX customer support team.                 *)
(*                                                         *)
(***********************************************************)
(* GENERAL FUNCTION LIST -                                 *)
(*   POWER, STATUS, TRANSPORTS (PL,REV.PL,ST,PSE,FF,REW,   *)
(*   SEARCHES,REC,FRAME ADV.)                              *)
(***********************************************************)
(*          DEVICE NUMBER DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_DEVICE

VCR    = 1                             (* AXC-232: PANASONIC AG-60400 *)
PNL    = 128

(***********************************************************)
(*               CONSTANT DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_CONSTANT

STX = $02
ETX = $03

(* BUFFER MODE VALUES *)
TRANSPORT_STATE = 1
REQUEST_DECK_ID = 2

(***********************************************************)
(*               VARIABLE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_VARIABLE

VCR_BUFFER[100]    (* BUFFER VARIABLE *)
BUFFER_MODE        (* TRACKS WHAT IS EXPECTED IN THE BUFFER *)
DECK_ID[10]        (* STRING: WHICH DECK MODEL *)
TEMP_STR[50]       (* STRING *)

(***********************************************************)
(*               LATCHING DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_LATCHING

(***********************************************************)
(*       MUTUALLY EXCLUSIVE DEFINITIONS GO BELOW           *)
(***********************************************************)
DEFINE_MUTUALLY_EXCLUSIVE

(***********************************************************)
(*           SUBROUTINE DEFINITIONS GO BELOW               *)
(***********************************************************)

(* THIS ROUTINE ROUTINE DETERMINES THE TRANSPORT STATE     *)
(* FROM THE TEXT OF "CMD", AND THEN SETS THE APPROPRIATE   *)
(* FEEDBACK CHANNELS OF THE DEVICE NUMBER OF "CARD".       *)
DEFINE_CALL 'GET TRANSPORT STATE' (CARD,CMD[20])
LOCAL_VAR CH[2]
{
  CH = REMOVE_STRING(CMD,"STX",1)
  SELECT
  {
    ACTIVE (LEFT_STRING(CMD,3) = 'OPL'):    (* PLAY *)
      SYSTEM_CALL 'FEEDBACK' (CARD,1,0)
    ACTIVE (LEFT_STRING(CMD,3) = 'OSP'):    (* STOP *)
      SYSTEM_CALL 'FEEDBACK' (CARD,2,0)
    ACTIVE ((LEFT_STRING(CMD,3) = 'OPA') OR (* PAUSE *)
            (LEFT_STRING(CMD,3) = 'OPP')):  (* STILL *)
      SYSTEM_CALL 'FEEDBACK' (CARD,3,0)
    ACTIVE (LEFT_STRING(CMD,3) = 'OFF'):    (* FAST-FORWARD *)
      SYSTEM_CALL 'FEEDBACK' (CARD,4,0)
    ACTIVE (LEFT_STRING(CMD,3) = 'ORW'):    (* REWIND *)
      SYSTEM_CALL 'FEEDBACK' (CARD,5,0)
    ACTIVE (LEFT_STRING(CMD,3) = 'OSF'):    (* SEARCH FORWARD *)
      SYSTEM_CALL 'FEEDBACK' (CARD,6,0)
    ACTIVE (LEFT_STRING(CMD,3) = 'OSR'):    (* SEARCH REVERSE *)
      SYSTEM_CALL 'FEEDBACK' (CARD,7,0)
    ACTIVE (LEFT_STRING(CMD,3) = 'ORC'):    (* RECORD *)
      SYSTEM_CALL 'FEEDBACK' (CARD,8,0)
    ACTIVE (LEFT_STRING(CMD,3) = 'ORP'):    (* RECORD PAUSE *)
    {
      SYSTEM_CALL 'FEEDBACK' (CARD,8,0)
      ON[CARD,3]
    }
    (* OTHER RESULTS *)
    ACTIVE (LEFT_STRING(CMD,3) = 'OPR'):    (* REVERSE PLAY *)
    {
    }
    ACTIVE (LEFT_STRING(CMD,3) = 'POF'):    (* POWER OFF *)
    {
    }
    ACTIVE (LEFT_STRING(CMD,3) = 'PON'):    (* POWER ON *)
    {
    }
    ACTIVE (LEFT_STRING(CMD,3) = 'OEJ'):    (* TAPE EJECTED *)
    {
    }
  }
}

DEFINE_CALL 'GET DECK ID' (ID[10],CMD[20])
LOCAL_VAR CH[2]
{
  CH = REMOVE_STRING(CMD,"STX",1)
  ID = LEFT_STRING(CMD,(LENGTH_STRING(CMD)-1))
}

DEFINE_CALL 'SEND TO VCR' (CARD,CMD[20])
{
  SEND_STRING CARD,"STX,CMD,ETX"
  SEND_STRING 0,"CMD,$0D,$0A"
}

(* SYSTEM_CALL-LIKE DEFINITION *)
DEFINE_CALL 'AG-6040 VCR' (CARD,PANEL,PLAYB,STOPB,PAUSEB,
                           FFWDB,REWB,SFWDB,SREVB,RECB)
{
  PUSH[PANEL,PLAYB]                    (* PLAY *)
    IF (NOT [CARD,248])
    {
      CALL 'SEND TO VCR' (CARD,'OPL')
      SYSTEM_CALL 'FEEDBACK' (CARD,1,0)
    }

  PUSH[PANEL,STOPB]                    (* STOP *)
  {
    CALL 'SEND TO VCR' (CARD,'OSP')
    SYSTEM_CALL 'FEEDBACK' (CARD,2,0)
  }

  PUSH[PANEL,PAUSEB]                   (* PAUSE *)
    IF (NOT[CARD,242])
    {
      CALL 'SEND TO VCR' (CARD,'OPA')
      IF ([CARD,243] AND [CARD,248])
        OFF[CARD,243]
      ELSE IF ([CARD,248])
        ON[CARD,243]
      ELSE IF ([CARD,243])
        SYSTEM_CALL 'FEEDBACK' (CARD,1,0)
      ELSE
        SYSTEM_CALL 'FEEDBACK' (CARD,3,0)
    }

  PUSH[PANEL,FFWDB]                    (* FAST-FORWARD *)
    IF (NOT [CARD,248])
    {
      IF ((SFWDB = 0) AND
          ([CARD,241] OR [CARD,243] OR [CARD,246] OR [CARD,247]))
      {
        CALL 'SEND TO VCR' (CARD,'OSF:7')
        SYSTEM_CALL 'FEEDBACK' (CARD,6,0)
      }
      ELSE
      {
        CALL 'SEND TO VCR' (CARD,'OFF')
        SYSTEM_CALL 'FEEDBACK' (CARD,4,0)
      }
    }

  PUSH[PANEL,REWB]                     (* REWIND *)
    IF (NOT [CARD,248])
    {
      IF ((SREVB = 0) AND
          ([CARD,241] OR [CARD,243] OR [CARD,246] OR [CARD,247]))
      {
        CALL 'SEND TO VCR' (CARD,'OSR:7')
        SYSTEM_CALL 'FEEDBACK' (CARD,7,0)
      }
      ELSE
      {
        CALL 'SEND TO VCR' (CARD,'ORW')
        SYSTEM_CALL 'FEEDBACK' (CARD,5,0)
      }
    }

  PUSH[PANEL,SFWDB]                    (* SEARCH FORWARD *)
    IF (NOT [CARD,248])
    {
      CALL 'SEND TO VCR' (CARD,'OSF:7')
      SYSTEM_CALL 'FEEDBACK' (CARD,6,0)
    }

  PUSH[PANEL,SREVB]                    (* SEARCH REVERSE *)
    IF (NOT [CARD,248])
    {
      CALL 'SEND TO VCR' (CARD,'OSR:7')
      SYSTEM_CALL 'FEEDBACK' (CARD,7,0)
    }

  PUSH[PANEL,RECB]                     (* RECORD *)
  {
    CALL 'SEND TO VCR' (CARD,'ORC')
    IF ([CARD,243] AND NOT[CARD,248])
      ON[CARD,248]
    ELSE
      SYSTEM_CALL 'FEEDBACK' (CARD,8,0)
  }

  [PANEL,PLAYB]  = [CARD,241]
  [PANEL,STOPB]  = [CARD,242]
  [PANEL,PAUSEB] = [CARD,243]
  [PANEL,FFWDB]  = ([CARD,244] OR ([CARD,246] AND (SFWDB = 0)))
  [PANEL,REWB]   = ([CARD,245] OR ([CARD,247] AND (SREVB = 0)))
  [PANEL,SFWDB]  = [CARD,246]
  [PANEL,SREVB]  = [CARD,247]
  [PANEL,RECB]   = [CARD,248]
}

(***********************************************************)
(*                STARTUP CODE GOES BELOW                  *)
(***********************************************************)
DEFINE_START

CREATE_BUFFER VCR,VCR_BUFFER

BUFFER_MODE = TRANSPORT_STATE
CALL 'SEND TO VCR' (VCR,'QOP')

(***********************************************************)
(*            THE ACTUAL PROGRAM GOES BELOW                *)
(***********************************************************)
DEFINE_PROGRAM

(*** BUFFER PROCESSING ***)
IF (LENGTH_STRING(VCR_BUFFER))
{
  IF (FIND_STRING(VCR_BUFFER,"ETX",1))
  {
    TEMP_STR = REMOVE_STRING(VCR_BUFFER,"ETX",1)
    IF (BUFFER_MODE = TRANSPORT_STATE)
      CALL 'GET TRANSPORT STATE' (VCR,TEMP_STR)
    ELSE IF (BUFFER_MODE = REQUEST_DECK_ID)
      CALL 'GET DECK ID' (DECK_ID,TEMP_STR)
  }
  ELSE
    VCR_BUFFER = ""
}


(*** SYSTEM CALL FORMAT ***)
CALL 'AG-6040 VCR' (VCR,PNL,1,2,3,4,5,0,0,8)


(*** STANDARD TRANSPORT CONTROL ***)
PUSH[PNL,11]                           (* PLAY *)
{
  BUFFER_MODE = TRANSPORT_STATE
  CALL 'SEND TO VCR' (VCR,'OPL')
}
[PNL,11] = [VCR,241]

PUSH[PNL,12]                           (* STOP *)
{
  BUFFER_MODE = TRANSPORT_STATE
  CALL 'SEND TO VCR' (VCR,'OSP')
}
[PNL,12] = [VCR,242]

PUSH[PNL,13]                           (* PAUSE/STILL *)
{
  BUFFER_MODE = TRANSPORT_STATE
  CALL 'SEND TO VCR' (VCR,'OPA')
}
[PNL,13] = [VCR,243]

PUSH[PNL,14]                           (* FAST-FORWARD *)
{
  BUFFER_MODE = TRANSPORT_STATE
  CALL 'SEND TO VCR' (VCR,'OFF')
}
[PNL,14] = [VCR,244]

PUSH[PNL,15]                           (* REWIND *)
{
  BUFFER_MODE = TRANSPORT_STATE
  CALL 'SEND TO VCR' (VCR,'ORW')
}
[PNL,15] = [VCR,245]

PUSH[PNL,16]                           (* SEARCH FORWARD (CUE) *)
{
  BUFFER_MODE = TRANSPORT_STATE
  CALL 'SEND TO VCR' (VCR,'OSF:7')
}
[PNL,16] = [VCR,246]

PUSH[PNL,17]                           (* SEARCH REVERSE (REVIEW) *)
{
  BUFFER_MODE = TRANSPORT_STATE
  CALL 'SEND TO VCR' (VCR,'OSR:7')
}
[PNL,17] = [VCR,247]

PUSH[PNL,18]                           (* RECORD *)
{
  BUFFER_MODE = TRANSPORT_STATE
  CALL 'SEND TO VCR' (VCR,'ORC')
}
[PNL,18] = [VCR,248]


(*** OTHER TRANSPORT COMMANDS ***)
PUSH[PNL,21]                           (* REVERSE PLAY *)
{
  BUFFER_MODE = TRANSPORT_STATE
  CALL 'SEND TO VCR' (VCR,'OPR')
}

PUSH[PNL,22]                           (* FORWARD ADVANCE *)
{
  BUFFER_MODE = TRANSPORT_STATE
  CALL 'SEND TO VCR' (VCR,'OAF')
}

PUSH[PNL,23]                           (* REVERSE ADVANCE *)
{
  BUFFER_MODE = TRANSPORT_STATE
  CALL 'SEND TO VCR' (VCR,'OAR')
}

PUSH[PNL,33]                           (* EJECT *)
{
  BUFFER_MODE = TRANSPORT_STATE
  CALL 'SEND TO VCR' (VCR,'OEJ')
}


(*** POWER ***)
PUSH[PNL,31]                           (* POWER ON *)
{
  BUFFER_MODE = TRANSPORT_STATE
  CALL 'SEND TO VCR' (VCR,'PON')
}

PUSH[PNL,32]                           (* POWER OFF *)
{
  BUFFER_MODE = TRANSPORT_STATE
  CALL 'SEND TO VCR' (VCR,'POF')
}


(*** MISCELLANEOUS ***)
PUSH[PNL,36]                           (* SET TIME *)
{
  TEMP_STR = TIME
  CALL 'SEND TO VCR' (VCR,"'TTS:',TEMP_STR[1],TEMP_STR[2],TEMP_STR[4],
                                  TEMP_STR[5],TEMP_STR[7],TEMP_STR[8]")
}

PUSH[PNL,37]                           (* SET DATE *)
{
  SYSTEM_CALL 'LONGDATE' (TEMP_STR)
  CALL 'SEND TO VCR' (VCR,"'TCS:',TEMP_STR[7],TEMP_STR[8],TEMP_STR[9],TEMP_STR[10],
                                  TEMP_STR[1],TEMP_STR[2],TEMP_STR[4],TEMP_STR[5]")
}


(*** STATUS REQUESTS ***)
PUSH[PNL,41]                           (* REQUEST TRANSPORT STATE *)
{
  BUFFER_MODE = TRANSPORT_STATE
  CALL 'SEND TO VCR' (VCR,'QOP')
}

PUSH[PNL,42]                           (* REQUEST DECK ID (MODEL) *)
{
  BUFFER_MODE = REQUEST_DECK_ID
  CALL 'SEND TO VCR' (VCR,'QID')
}


(***********************************************************)
(*                     END OF PROGRAM                      *)
(*        DO NOT PUT ANY CODE BELOW THIS COMMENT           *)
(***********************************************************)

